"""XML as lists and dictionaries, via cElementTree objects.

Based on: http://code.activestate.com/recipes/573463/

Namespace removal cleanups made locally.
"""

from xml.etree import cElementTree

import re


def _f(tag):
    """Remove XML namespaces from text (as introduced by ElementTree)."""
    if tag.startswith('{'):
        # Strip the namespace from the beginning of the tag
        return tag[tag.find('}')+1:]
    else:
        return tag


class AttrDict(dict):
    """A dictionary with attribute access for keys."""

    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)

    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, name, value):
        self[name] = value


def _convert_xml_to_dict_recurse(node, dictclass):
    nodedict = dictclass()

    if len(node.items()) > 0:
        # if we have attributes, set them
        nodedict.update(dict(node.items()))

    for child in node:
        # recursively add the element's children
        newitem = _convert_xml_to_dict_recurse(child, dictclass)
        if nodedict.has_key(_f(child.tag)):
            # found duplicate tag, force a list
            if type(nodedict[_f(child.tag)]) is type([]):
                # append to existing list
                nodedict[_f(child.tag)].append(newitem)
            else:
                # convert to list
                nodedict[_f(child.tag)] = [nodedict[_f(child.tag)], newitem]
        else:
            # only one, directly set the dictionary
            nodedict[_f(child.tag)] = newitem

    if node.text is None:
        text = ''
    else:
        text = node.text.strip()

    if len(nodedict) > 0:
        # if we have a dictionary add the text as a dictionary value
        # (if there is any)
        if len(text) > 0:
            nodedict['_text'] = text
    else:
        # if we don't have child nodes or attributes, just set the text
        nodedict = text
    return nodedict


def convert_xml_to_dict(root, dictclass=AttrDict):
    """Converts an XML string or ElementTree Element to a dictionary."""
    # If a string is passed in, try to parse it
    if type(root) == type(''):
        root = cElementTree.fromstring(root)

    result = dictclass({_f(root.tag): _convert_xml_to_dict_recurse(
        root, dictclass)})
    return result
